##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from datetime import datetime
import crm
import time
from tools.translate import _
from crm.crm import crm_case
import binascii
import tools
import logging

AVAILABLE_STATES = [
    ('draft', 'Draft'),
    ('open', 'Open'),
    ('cancel', 'Cancelled'),
    ('done', 'Done'),
    ('pending', 'Pending'),
    ('ready', 'Intake OK'),
    ('planned', 'Planned'),
]


class crm_case_stage(osv.osv):
    """ Stage of case """
    _inherit = "crm.case.stage"

    def _get_type_value(self, cr, user, context):
        return [('lead','Lead'),('opportunity','Opportunity'),('activity','Activity')]

    _columns = {
        'type': fields.selection(_get_type_value, 'Type'),
    }

crm_case_stage()

class crm_activities(crm_case, osv.osv):
    """ CRM Activity Case """
    _name = "crm.activities"
    _description = "Case Driven Activities"
    _order = "date_action, priority, id desc"
    _inherit = ['mailgate.thread','res.partner.address','crm.lead']
	
    def _get_stage_id(self, cr, uid, context={}):
        if context is None:
            context = {}
        stage_pool = self.pool.get('crm.case.stage')
        stage_type = context and context.get('stage_type','')
        logging.getLogger('crm_enhancements').warning("Stage Type %s", stage_type)
        if not stage_type:
            stage_type='activity'
        domain = [('type', '=', stage_type)]
        stages = stage_pool.search(cr, uid, domain, order='sequence')
        return stages[0]

    def action_makeMeeting(self, cr, uid, ids, context=None):
        logging.getLogger('crm_activities').warning("Context %s", context)
        """
        This opens Meeting's calendar view to schedule meeting on current Opportunity
        @param self: The object pointer

        @param cr: the current row, from the database cursor,
        @param uid: the current user's ID for security checks,
        @param ids: List of Opportunity to Meeting IDs
        @param context: A standard dictionary for contextual values

        @return : Dictionary value for created Meeting view
        """
        value = {}
        for opp in self.browse(cr, uid, ids, context=context):
            data_obj = self.pool.get('ir.model.data')

            # Get meeting views
            result = data_obj._get_id(cr, uid, 'crm', 'view_crm_case_meetings_filter')
            res = data_obj.read(cr, uid, result, ['res_id'])
            id1 = data_obj._get_id(cr, uid, 'crm_activities', 'crm_case_calendar_view_meet')
            id2 = data_obj._get_id(cr, uid, 'crm_activities', 'crm_case_form_view_meet')
            id3 = data_obj._get_id(cr, uid, 'crm_activities', 'crm_case_tree_view_meet')
            if id1:
                id1 = data_obj.browse(cr, uid, id1, context=context).res_id
            if id2:
                id2 = data_obj.browse(cr, uid, id2, context=context).res_id
            if id3:
                id3 = data_obj.browse(cr, uid, id3, context=context).res_id

            context = {
                'default_opportunity_id': opp.id,
                'default_partner_id': opp.partner_id and opp.partner_id.id or False,
                'default_user_id': context['uid'],
                'default_section_id': opp.section_id and opp.section_id.id or False,
                'default_email_from': opp.email_from,
                'default_state': 'open',
                'default_name': opp.name
            }
            logging.getLogger('crm_activities').warning("Context 2: %s", context)
            value = {
                'name': _('Meetings'),
                'context': context,
                'view_type': 'form',
                'view_mode': 'calendar,form,tree',
                'res_model': 'crm.meeting.activities',
                'view_id': False,
                'views': [(id1, 'calendar'), (id2, 'form'), (id3, 'tree')],
                'type': 'ir.actions.act_window',
                'search_view_id': res['res_id'],
                'nodestroy': True
            }
        return value

    def _compute_day(self, cr, uid, ids, fields, args, context=None):
        """
        @param cr: the current row, from the database cursor,
        @param uid: the current user's ID for security checks,
        @param ids: List of Openday's IDs
        @return: difference between current date and log date
        @param context: A standard dictionary for contextual values
        """
        cal_obj = self.pool.get('resource.calendar')
        res_obj = self.pool.get('resource.resource')

        res = {}
        for lead in self.browse(cr, uid, ids, context=context):
            for field in fields:
                res[lead.id] = {}
                duration = 0
                ans = False
                if field == 'day_open':
                    if lead.date_open:
                        date_create = datetime.strptime(lead.create_date, "%Y-%m-%d %H:%M:%S")
                        date_open = datetime.strptime(lead.date_open, "%Y-%m-%d %H:%M:%S")
                        ans = date_open - date_create
                        date_until = lead.date_open
                elif field == 'day_close':
                    if lead.date_closed:
                        date_create = datetime.strptime(lead.create_date, "%Y-%m-%d %H:%M:%S")
                        date_close = datetime.strptime(lead.date_closed, "%Y-%m-%d %H:%M:%S")
                        date_until = lead.date_closed
                        ans = date_close - date_create
                if ans:
                    resource_id = False
                    if lead.user_id:
                        resource_ids = res_obj.search(cr, uid, [('user_id','=',lead.user_id.id)])
                        if len(resource_ids):
                            resource_id = resource_ids[0]

                    duration = float(ans.days)
                    if lead.section_id and lead.section_id.resource_calendar_id:
                        duration =  float(ans.days) * 24
                        new_dates = cal_obj.interval_get(cr,
                            uid,
                            lead.section_id.resource_calendar_id and lead.section_id.resource_calendar_id.id or False,
                            datetime.strptime(lead.create_date, '%Y-%m-%d %H:%M:%S'),
                            duration,
                            resource=resource_id
                        )
                        no_days = []
                        date_until = datetime.strptime(date_until, '%Y-%m-%d %H:%M:%S')
                        for in_time, out_time in new_dates:
                            if in_time.date not in no_days:
                                no_days.append(in_time.date)
                            if out_time > date_until:
                                break
                        duration =  len(no_days)
                res[lead.id][field] = abs(int(duration))
        return res

    def onchange_section_id(self, cr, uid, ids, section_id, context={}):
        v = {}
        logging.getLogger('crm_enhancements').warning("Value sent %s", section_id)
        if section_id <> 0 :
            sid = self.pool.get('crm.case.section').browse(cr,uid,section_id)
            v = { 'user_id':sid.user_id.id }
        return {'value':v}

    def _get_attach_ids(self, cr, uid, ids, field_name, arg, context={}):
        result = {}
        for record in self.browse(cr, uid, ids, context=context):
            if record.message_ids:
                result[record.id]=[]
                for msgid in record.message_ids:
                    for attachid in msgid.attachment_ids:
                        result[record.id].append(attachid.id)
            else:
                result[record.id]= False
        return result

    def _get_comms_ids(self, cr, uid, ids, field_name, arg, context={}):
        result = {}
        for record in self.browse(cr, uid, ids, context=context):
            if record.message_ids:
                result[record.id]=[]
                for msgid in record.message_ids:
                    if ( msgid.history or msgid.name.upper() == "NOTE" ) :
                        if ( field_name=='communication_ids'):
                            result[record.id].append(msgid.id)
                    elif ( field_name=='history_ids'):
                        result[record.id].append(msgid.id)
            else:
                result[record.id]= False
        return result

    def case_ready(self, cr, uid, ids, *args):
        """Readies Case
        @param self: The object pointer
        @param cr: the current row, from the database cursor,
        @param uid: the current user's ID for security checks,
        @param ids: List of case Ids
        @param *args: Tuple Value for additional Params
        """
        cases = self.browse(cr, uid, ids)
        cases[0].state # to fill the browse record cache
        self._history(cr, uid, cases, _('Ready'))
        self.write(cr, uid, ids, {'state': 'ready'})
        #
        # We use the cache of cases to keep the old case state
        #
        self._action(cr, uid, cases, 'ready')
        return True

    def case_planned(self, cr, uid, ids, *args):
        """Case is effectively planned
        @param self: The object pointer
        @param cr: the current row, from the database cursor,
        @param uid: the current user's ID for security checks,
        @param ids: List of case Ids
        @param *args: Tuple Value for additional Params
        """
        cases = self.browse(cr, uid, ids)
        cases[0].state # to fill the browse record cache
        self._history(cr, uid, cases, _('Planned'))
        self.write(cr, uid, ids, {'state': 'planned'})
        #
        # We use the cache of cases to keep the old case state
        #
        self._action(cr, uid, cases, 'planned')
        return True
    
    _columns = {
        # Overridden from res.partner.address:
        'partner_id': fields.many2one('res.partner', 'Partner', ondelete='set null',
            select=True, help="Optional linked partner, usually after conversion of the lead"),
        'partner_address_id': fields.many2one('res.partner.address', 'Partner Contact', \
                                 domain="[('partner_id','=',partner_id)]"),

        # From crm.case
        'id': fields.integer('ID'),
        'name': fields.char('Name', size=64),
        'active': fields.boolean('Active', required=False),
        'date_action_last': fields.datetime('Last Action', readonly=1),
        'date_action_next': fields.datetime('Next Action', readonly=1),
        'email_from': fields.char('Email', size=128, help="E-mail address of the contact"),
        'section_id': fields.many2one('crm.case.section', 'Section', \
                        select=True, help='Sales team to which this case belongs to. Defines responsible user and e-mail address for the mail gateway.'),
        'create_date': fields.datetime('Creation Date' , readonly=True),
        'email_cc': fields.text('Global CC', size=252 , help="These email addresses will be added to the CC field of all inbound and outbound emails for this record before being sent. Separate multiple email addresses with a comma"),
        'description': fields.text('Notes'),
        'write_date': fields.datetime('Update Date' , readonly=True),

        # Lead fields
        'categ_id': fields.many2one('crm.case.categ', 'Category', \
            domain="['|',('section_id','=',section_id),('section_id','=',False), ('object_id.model', '=', 'crm.lead')]"),
        'type_id': fields.many2one('crm.case.resource.type', 'Campaign', \
            domain="['|',('section_id','=',section_id),('section_id','=',False)]"),
        'channel_id': fields.many2one('res.partner.canal', 'Channel'),

        'contact_name': fields.char('Contact Name', size=64),
        'partner_name': fields.char("Customer Name", size=64,help='The name of the future partner that will be created while converting the into opportunity'),
        'optin': fields.boolean('Opt-In', help="If opt-in is checked, this contact has accepted to receive emails."),
        'optout': fields.boolean('Opt-Out', help="If opt-out is checked, this contact has refused to receive emails or unsubscribed to a campaign."),
        'type':fields.selection([
            ('activity','Activity'),
        ],'Type', help="Type is used to separate different types of production entities"),
        'priority': fields.selection(crm.crm.AVAILABLE_PRIORITIES, 'Priority'),
        'date_closed': fields.datetime('Closed', readonly=True),
        'stage_id': fields.many2one('crm.case.stage', 'Stage', domain="[('type','=','lead')]"),
        'user_id': fields.many2one('res.users', 'Responsible',help='By Default Salesman is Administrator when create New User'),
        'engineer_id': fields.many2one('res.users', 'Planned for',help='The appointed engineer to execute the call'),
        'referred': fields.char('Referred By', size=64),
        'date_open': fields.datetime('Opened', readonly=True),
        'day_open': fields.function(_compute_day, string='Days to Open', \
                                method=True, multi='day_open', type="float", store=True),
        'day_close': fields.function(_compute_day, string='Days to Close', \
                                method=True, multi='day_close', type="float", store=True),
        'state': fields.selection(AVAILABLE_STATES, 'State', size=16, readonly=True,
                                  help='The state is set to \'Draft\', when a case is created.\
                                  \nIf the case is in progress the state is set to \'Open\'.\
                                  \nWhen the case is over, the state is set to \'Done\'.\
                                  \nIf the case needs to be reviewed then the state is set to \'Pending\'.'),
        'message_ids': fields.one2many('mailgate.message', 'res_id', 'Messages', domain=[('model','=',_name)]),

        # Opportunity fields
        'probability': fields.float('Probability (%)',group_operator="avg"),
        'planned_revenue': fields.float('Expected Revenue'),
        'ref': fields.reference('Reference', selection=crm.crm._links_get, size=128),
        'ref2': fields.reference('Reference 2', selection=crm.crm._links_get, size=128),
        'phone': fields.char("Phone", size=64),
        'date_deadline': fields.date('Expected Closing'),
        'date_action': fields.date('Next Action Date'),
        'title_action': fields.char('Next Action', size=64),

        # CRM enhancements fields
        'attachment_ids' : fields.function(_get_attach_ids,method=True,type="many2many",string="Attachments",relation="ir.attachment"),
        'communication_ids' : fields.function(_get_comms_ids,method=True,type="many2many",string="Communication Items",relation="mailgate.message"),
        'history_ids' : fields.function(_get_comms_ids,method=True,type="many2many",string="History Items",relation="mailgate.message"),

    }


    _defaults = {
        'active': lambda *a: 1,
        'user_id': crm_case._get_default_user,
        'email_from': crm_case._get_default_email,
        'state': lambda *a: 'open',
        'type': lambda *a: 'activity',
        'section_id': crm_case._get_section,
        'company_id': lambda s, cr, uid, c: s.pool.get('res.company')._company_default_get(cr, uid, 'crm.lead', context=c),
        'priority': lambda *a: crm.crm.AVAILABLE_PRIORITIES[2][0],
        'stage_id': _get_stage_id,
    }

    def message_new(self, cr, uid, msg, context=None):
        """
        Automatically calls when new email message arrives

        @param self: The object pointer
        @param cr: the current row, from the database cursor,
        @param uid: the current user's ID for security checks
        """
        mailgate_pool = self.pool.get('email.server.tools')
        subject = msg.get('subject')
        body = msg.get('body')
        msg_from = msg.get('from')
        priority = msg.get('priority')

        vals = {
            'name': subject,
            'email_from': msg_from,
            'email_cc': msg.get('cc'),
            'description': body,
            'user_id': False,
        }
        if msg.get('priority', False):
            vals['priority'] = priority

        res = mailgate_pool.get_partner(cr, uid, msg.get('from') or msg.get_unixfrom())
        if res:
            vals.update(res)
        res = self.create(cr, uid, vals, context)
        attachments = msg.get('attachments', [])
        for attachment in attachments or []:
            data_attach = {
                'name': attachment,
                'datas':binascii.b2a_base64(str(attachments.get(attachment))),
                'datas_fname': attachment,
                'description': 'Mail attachment',
                'res_model': self._name,
                'res_id': res,
            }
            self.pool.get('ir.attachment').create(cr, uid, data_attach)

        return res

crm_activities()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:        
